@@ -22,12 +22,13 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 import os
 import re
 import urllib
+from urlparse import unquote
 from collections import namedtuple
 from posixpath import normpath
 
 
 __all__ = ["parse", "extract", "split", "split_netloc", "assemble", "encode",
-           "normalize", "normalize_path"]
+           "normalize", "normalize_path", "normalize_path2"]
 
 
 PSL_URL = 'http://mxr.mozilla.org/mozilla-central/source/netwerk/dns/effective_tld_names.dat?raw=1'
@@ -66,7 +67,7 @@ ParseResult = namedtuple('ParseResult', ['scheme', 'username', 'password',
 def normalize(url):
     """Normalize a URL
     """
-    parts = extract(url)
+    parts = extract(url.strip())
     return assemble(parts, default_path='/')
 
 
@@ -113,10 +114,28 @@ def assemble(parts, default_path=''):
 def normalize_path(path):
     """Normalize path (collapse etc.)
     """
-    npath = normpath(path)
-    if path.endswith('/') and not npath.endswith('/'):
-        npath += '/'
-    return npath.replace('//', '/')
+    if path == '//':
+        return '/'
+    return normpath(unquote(path))
+
+
+def normalize_path2(path):
+    parts = path.split('/')
+    while 1:
+        parts = filter(None, parts)
+        for i in range(len(parts)):
+            if parts[i] == '.':
+                parts[i] = ''
+                break
+            if parts[i] == '..':
+                parts[i-1] = ''
+                parts[i] = '' 
+                break
+        else:
+            break
+    result = '/' if path[0] == '/' else ''
+    result += '/'.join(parts)
+    return result
 
 
 def split(url):
